/*
 * labjack - PD External for LabJack Cards
 * 
 * Ecole Nationale Superieure d'Ingenieurs de Bretagne Sud
 * Copyright (C) 2012 Florent de Lamotte
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version. (see LICENCE.txt)
 *
 */

#include "m_pd.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <pthread.h>
#include <semaphore.h>

#include "labjackusb.h"
#include <libusb-1.0/libusb.h>

#include "labjack.h"
#include "LJ_AOOut.h"


// Demonstrates how to build the AIStream packet.
void LJ_buildAOSampleBytes(char * sendBuffer, t_float ao0, t_float ao1);

static void *LJ_AOOut_setup_thread(void *w)
{
    	t_LJ_AOOut *x = (t_LJ_AOOut*) w;

    	// Setup the variables we will need.
    	int r = 0; // For checking return values
    	BYTE sendBuffer[U12_COMMAND_LENGTH];
    	BYTE recBuffer[U12_COMMAND_LENGTH];

	int i;

    	while(1) {
    		sem_wait(&x->go);

    		// Builds the AISample command
    		LJ_buildAOSampleBytes(sendBuffer, x->ao0, x->ao1);
    
    		// Write the command, and read the response.
    		r = writeRead(devHandle, sendBuffer, recBuffer );
    
    		sem_post(&x->busy);     // on libere le thread
    	}

}

static void LJ_AOOut_thread_start(t_LJ_AOOut *x)
{
	// create the worker thread
    	if(pthread_attr_init(&x->LJ_AOOut_thread_attr) < 0){
       		error("labjack: could not launch acquire thread");
       		return;
    	} else if(pthread_attr_setdetachstate(&x->LJ_AOOut_thread_attr, PTHREAD_CREATE_DETACHED) < 0) {
	   	error("labjack: could not launch acquire thread");
	   	return;
    	}

    	if(pthread_create(&x->x_threadid, &x->LJ_AOOut_thread_attr, LJ_AOOut_setup_thread, x) < 0) {
	   	error("labjack: could not launch aquire thread");
       		return;
    	}
}


/*
 * Executed when banged ... take a shot ;)
 */
void LJ_AOOut_sample(t_LJ_AOOut *x)
{
	if (sem_trywait(&(x->busy)) != -1) {	
		sem_post(&x->go);
	} else {
		outlet_bang(x->err);
	}
}

void *LJ_AOOut_new(t_symbol *s, int argc, t_atom *argv)
{
	t_LJ_AOOut *x;	
	x = (t_LJ_AOOut *)pd_new(LJ_AOOut_class);
	int i;
	
	// inlets
	floatinlet_new (&x->p_ob, &x->ao0);
	floatinlet_new (&x->p_ob, &x->ao1);

	x->err = outlet_new(&x->p_ob, &s_bang);
	x->errors = 0;

	// setting up thread and semaphores
	sem_init (&x->busy, 0, 1);
	sem_init (&x->go, 0, 0);

	LJ_AOOut_thread_start(x);

	return x; 
}

void LJ_AOOut_free(t_LJ_AOOut *x)
{
	while(pthread_cancel(x->x_threadid) < 0);	
    	//Close the device.

    	//LJUSB_CloseDevice(x->devHandle);
}

// Uses information from section 5.4 of the U12 User's Guide to make a AISample
// packet.
// http://labjack.com/support/u12/users-guide/5.4
void LJ_buildAOSampleBytes(char * sendBuffer, t_float ao0, t_float ao1) {

    	int ao0_out;
	int ao1_out;

	ao0_out = (ao0 / 5.0) * 0x3FF;
	ao1_out = (ao1 / 5.0) * 0x3FF;

	// Build up the bytes
    	sendBuffer[0] = 0;
    	sendBuffer[1] = 0;
    	sendBuffer[2] = 0;
    	sendBuffer[3] = 0;
    	sendBuffer[4] = 0;
    	sendBuffer[5] = ((ao0_out & 3) << 2) | (ao1_out & 3);
    	sendBuffer[6] = ao0_out >> 2;
    	sendBuffer[7] = ao1_out >> 2;
    
}
