
/*
 * labjack - PD External for LabJack Cards
 * 
 * Ecole Nationale Superieure d'Ingenieurs de Bretagne Sud
 * Copyright (C) 2012 Florent de Lamotte
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version. (see LICENCE.txt)
 *
 */


#include "m_pd.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <pthread.h>
#include <semaphore.h>

#include <labjackusb.h>
#include <libusb-1.0/libusb.h>

#include "labjack.h"
#include "LJ_AISample.h"
#include "LJ_InputSample.h"
#include "LJ_AOOut.h"


/* valid for one Labjack */
HANDLE devHandle = NULL;
sem_t LJ_busy;


/*
 * run when importing library to pd
 * 
 * creates the classes for LJ objects
 */
int labjack_setup(void) 
{

	sem_init(&LJ_busy, 0, 1);

	/*
	 * Definition of LJ_AISample
	 */
	LJ_AISample_class = class_new ( gensym("LJ_AISample"),
					(t_newmethod)LJ_AISample_new, 0, sizeof(t_LJ_AISample), 	
					CLASS_DEFAULT, A_GIMME, 0);
	
	class_addbang(LJ_AISample_class, (t_method)LJ_AISample_sample);

	
	/*
	 * Definition of LJ_InputSample
	 */
	LJ_InputSample_class = class_new ( gensym("LJ_InputSample"),
					(t_newmethod)LJ_InputSample_new, 0, sizeof(t_LJ_InputSample), 	
					CLASS_DEFAULT, A_GIMME, 0);
	
	class_addbang(LJ_InputSample_class, (t_method)LJ_InputSample_sample);


	/*
	 * Definition of LJ_AOOut
	 */
	LJ_AOOut_class = class_new ( gensym("LJ_AOOut"),
					(t_newmethod)LJ_AOOut_new, 0, sizeof(t_LJ_AOOut), 
					CLASS_DEFAULT, A_GIMME, 0);
	
	class_addbang(LJ_AOOut_class, (t_method)LJ_AOOut_sample);

	post("labjack for pd version 0.1 (C) 2012 F. de Lamotte, ENSIBS", 0);
	return 1;
}


/*
 * Writes to Labjack and reads reply
 *
 * Automatically connects if disconnected
 * 
 * 
 * args :
 *  cur_handle : labjack handle
 *  sendBuffer : data sent to LabJack
 *  recBuffer  : received data
 *
 * returns : -1 on failure
 */
int writeRead(HANDLE cur_handle, BYTE * sendBuffer, BYTE * recBuffer ) 
{
    	int r = 0;

	sem_wait(&LJ_busy); // prevents concurrent use of the labjack

    	// Open the U12
	// TODO: Support also U3 and U6
	if (devHandle == NULL) { //open the labjack
		post("Opening Labjack");
   		devHandle = LJUSB_OpenDevice(1, 0, U12_PRODUCT_ID);
   		if( devHandle == NULL ) {
        		error("Couldn't open U12. Please connect one and try again.");
        		devHandle = 0;
			sem_post(&LJ_busy);
			return -1;
    		} else {
			post("Labjack opened");
		}
	}

    	r = LJUSB_Write( devHandle, sendBuffer, U12_COMMAND_LENGTH );
    	if( r != U12_COMMAND_LENGTH ) {
        	error("An error occurred when trying to write to the U12.");
		LJUSB_CloseDevice(devHandle);
		devHandle = NULL;
		sem_post(&LJ_busy);
		return -1;
    	}
    
    	r = LJUSB_Read( devHandle, recBuffer, U12_COMMAND_LENGTH );
   	if( r != U12_COMMAND_LENGTH ) {
        	if(errno == LIBUSB_ERROR_TIMEOUT) {
			error ("Timeout while talking to U12.");
    			r = LJUSB_Write( devHandle, sendBuffer, U12_COMMAND_LENGTH );

    			if( r == U12_COMMAND_LENGTH ) {
    				r = LJUSB_Read( devHandle, recBuffer, U12_COMMAND_LENGTH );
			}
        	}
        
		if (r != U12_COMMAND_LENGTH) {
       		 	error("An error occurred when trying to read from the U12.");// The error was: %d\n", errno);
        		LJUSB_CloseDevice(devHandle);
			devHandle = NULL;
			sem_post(&LJ_busy);
			return -1;
		}
    	}
    
	sem_post(&LJ_busy);

    	return 0;
}

